/**
 * <PRE>
 * Copyright and Permissions:
 *  Copyright Geoff Dale 1996, All rights reserved.
 *  You may freely redistribute this file as long as no modifications are made
 *     to the code or the comments (including this copyright).
 *  You may freely redistribute the compiled .class file created by this file.
 *  You may freely subclass from this class.
 *
 *
 * Naming Conventions:
 *  ClassName
 *  methodName
 *  void setVariableName( type a )  - setter accessor method
 *  type getVariableName()          - getter accessor method
 *  VARIABLE_NAME       - constants (static final)
 *  _VariableName       - (private or protected) instance variable
 *  s_VariableName      - static instance variable
 *  a_VariableName      - method argument
 *  a                   - setter argument
 *  variableName        - public instance or local variable
 *
 * History:
 *  gdale   10/02/96 - created.
 *
 * </PRE>
 * @author Geoff Dale <geoff1@home.com>
 * @version 0.1
 */

import java.io.PrintStream;

/**
 * <PRE>
 * Dynamic interface
 * </PRE>
 */
public interface DynamicPropertyInterface
{

//
// -- Public Methods --
//
    /**
     * @return true if object actually supports the property
     */
    public boolean setProperty( String a_Key, String a_Value );

    /**
     * @return the property; null if property doesn't exist
     */
    public String getProperty( String a_Key );

    /**
     * @return Two dimensional array of property info;
     *          null if no properties are supported.
     *         <BR> String[numKeys][3]
     *         <BR> String[i][0] is a key
     *         <BR> String[i][1] is the information,
     *              describing the property's use
     *         <BR> String[i][2] is one byte; flags defined by
     *              DynamicPropertyConstants.PROPERTIES_FLAG_ flags.
     * @see DynamicPropertyConstants
     */
    public String[][] getPropertyInfo();

    /**
     * @return Array of property keys supported
     *          null if no properties are supported.
     *         <BR> String[] = propertyKey
     */
    public String[] getPropertyKeys();

    /**
     * List properties, for debugging
     */
    public void list( PrintStream a_PrintStream );

}