// EzStego - a simple tool for steganography
//
// Copyright (C) 1996 by Romana Machado <romana@glamazon.com>.  All rights reserved.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
/** @(#) EzStego 2.0b3 by Romana Machado 11/96 */

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;

import ImageEncoder;
import GifEncoder;
import EzStegoEncoder;
import DynamicFilterInputStream;

//to get around image loading bug
import sun.awt.image.FileImageSource;

public class EzStego extends Applet implements Runnable, ImageConsumer
{
	
	private Image image = null;
	
	DynamicFilterInputStream cDynamicFilterInputStream;
	EndsWithFilter cEndsWithFilter;
	EzStegoEncoder cEzStegoEncoder;
	RGBPaletteSorter cRGBPaletteSorter;
	
	static final private boolean DEBUG = true;
	private static boolean NOGUI = false;
	private static boolean UNSTEG = false;
	private static boolean VERBOSE = false;
	
	static String inputfile, outputfile,imagefile, filtername = null;
	static final String versionString = " EzStego 2.0b3 by Romana Machado 11/96 ";
	static private EzStego stego;
	Image splashImage;

	private String imageDirString,imageFileString;
	private Panel buttonBox;
	StegoCanvas canvas;
	Label statusLabel;
    private Button openGIFButton;
	private Button stegButton;
	private Button unstegButton;
	private Button fetchButton;
	private Button urlButton;
	private TextField urlTextField;

    StegoPanel statusPanel;
    Color foreColor, errColor, buttonColor;
    ImageScrollbar horizontalSB, verticalSB;

	/*for getting color table*/
    ImageProducer   pro;
    int[] rgbPalette;
    int numRGBvalues;
    int bitDepth;
    
    private EzStegoEncoder ie;
    int spaceAvail;
    

	public EzStego()
	{		
		
		if (!NOGUI)
		{
			String  dfltFont = "TimesRoman";
	   		Font font = new Font(dfltFont,Font.PLAIN,12);
	      		foreColor = Color.black; 
	      		buttonColor = new Color(0xFFCCFF);
	        	errColor  = Color.red;
	       		 Color backColor = Color.white; 
			setLayout(new BorderLayout(0,0));
			setBackground(backColor);
			setForeground(foreColor);
			StegoPanel topBox = new StegoPanel();           //Set constraints for button box		            
			topBox.setBackground(backColor);
			topBox.setForeground(foreColor);
			add( "North",topBox);
			
			topBox.setLayout(new BorderLayout());
						
			Panel urlBox = new Panel();
			GridBagLayout urlBoxLayout = new GridBagLayout();
			GridBagConstraints gbc = new GridBagConstraints();
			
			urlBox.setLayout( urlBoxLayout );
			urlBox.setBackground(backColor);
			urlBox.setForeground(foreColor);

			Label urlLabel = new Label("URL:");
			urlLabel.setBackground(backColor);
			urlLabel.setForeground(foreColor);
			urlLabel.setAlignment(Label.LEFT);
			urlLabel.setFont(font);
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.NONE;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.weightx = 0;
			gbc.insets = new Insets(5,1,5,3);

			urlBox.add(urlLabel);
			urlBoxLayout.setConstraints(urlLabel,gbc);
						
			urlBox.add(urlTextField = new TextField());
			urlTextField.setEditable(true);
			urlTextField.minimumSize(50);
			urlTextField.preferredSize(50);
			urlTextField.setFont(new Font(dfltFont,Font.PLAIN,12));
			
			gbc.gridx = GridBagConstraints.RELATIVE;
		//	gbc.gridwidth = 4;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.weightx = 0.80;
			urlBox.add(urlTextField);
			urlBoxLayout.setConstraints(urlTextField,gbc);		
			
			urlButton = new Button("Fetch GIF");
			
			urlBox.add(urlButton);
			gbc.gridx = GridBagConstraints.RELATIVE;
		//	gbc.gridwidth = 4;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.NONE;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.weightx = 0.20;
			gbc.insets = new Insets(5,4,5,3);
			
			urlBox.add(urlButton);
			urlBoxLayout.setConstraints(urlButton,gbc);
			
			
			urlButton.setForeground(Color.black);
			urlButton.setBackground(buttonColor);
			urlButton.setFont(font);
			
			topBox.add("South",urlBox);

				
			buttonBox = new StegoPanel();
			GridBagLayout gbl = new GridBagLayout();
			buttonBox.setLayout(gbl);

			//GridBagConstraints 
			gbc = new GridBagConstraints();
			gbc.insets = new Insets(5,5,5,5);
			gbc.weighty = 1.0;
			gbc.weightx = 1.0;
			gbc.fill = GridBagConstraints.BOTH;
			
		
			buttonBox.setBackground(backColor);
			
			buttonBox.add(openGIFButton = new Button("Open GIF File"));
			openGIFButton.setForeground(Color.black);
			openGIFButton.setBackground(buttonColor);
			openGIFButton.setFont(font);
			gbl.setConstraints(openGIFButton,gbc);

			buttonBox.add(stegButton = new Button("Steg"));
			stegButton.setForeground(Color.black);
			stegButton.setBackground(buttonColor);
			stegButton.setFont(font);
			gbl.setConstraints(stegButton,gbc);
					
			buttonBox.add(unstegButton = new Button("Unsteg"));
			unstegButton.setForeground(Color.black);
			unstegButton.setBackground(buttonColor);
			unstegButton.setFont(font);
			gbl.setConstraints(unstegButton,gbc);
			
			
			topBox.add("North",buttonBox);
			
			
			//Set up canvas with scrollbars
			Panel canvasBox = new Panel();
			canvas = new StegoCanvas();
			canvas.setForeground(buttonColor);
			canvas.setBackground(Color.black);

			canvas.setFont(new Font(dfltFont,Font.PLAIN,12));

			horizontalSB = new ImageScrollbar(Scrollbar.HORIZONTAL, 
						0, 200, 0, 200,
						canvas);

			verticalSB = new ImageScrollbar(Scrollbar.VERTICAL, 
						0, 200, 0, 200,
						canvas);

			// Set the canvas & scrollbar layout
			GridBagLayout gridBag = new GridBagLayout();

			canvasBox.setLayout(gridBag);

			// first the canvas
			GridBagConstraints c = new GridBagConstraints();
			c.weightx = 1.0;
			c.weighty = 1.0;
			c.fill = GridBagConstraints.BOTH;
			gridBag.setConstraints(canvas, c);
			canvasBox.add(canvas);
			
			// then the horizontal Scrollbar
			c.weightx = 0.0;
			c.weighty = 0.0;
			c.gridx = 0;
			c.gridy = 1;
			gridBag.setConstraints(horizontalSB, c);
			canvasBox.add(horizontalSB);

			// finally, the vertical Scrollbar
			c.gridx = 1;
			c.gridy = 0;
			gridBag.setConstraints(verticalSB, c);
			canvasBox.add(verticalSB);

			add("Center",canvasBox);

			statusPanel = new StegoPanel();
			statusPanel.setBackground(backColor);
			statusPanel.setForeground(foreColor);
			gbl = new GridBagLayout();
			statusPanel.setLayout(gbl);
			
			statusLabel = new Label(versionString ,Label.LEFT);
			Font msgfont = new Font(dfltFont,Font.PLAIN,12);
			statusLabel.setFont(msgfont);
			statusLabel.setBackground(backColor);
			statusLabel.setForeground(foreColor);
			gbl.setConstraints(statusLabel,gbc);
			statusPanel.add(statusLabel);
			add("South", statusPanel);
			
			/*
			try
			{
				splashImage = Toolkit.getDefaultToolkit().getImage( "stegoIcon.gif" );		
			}
			catch (Exception e)
			{
				splashImage = null;
				}
			if (splashImage != null)
			{
				waitForImage(this,splashImage);
			}
			else
			{
				System.out.println( "Could not load splash image " );
			}
			*/
		
		}
	}
	
	public static void parseCmdLine(String args[])
	{
		int i = 0;
		String arg;
		
      while (i < args.length && args[i].startsWith("-"))
      {
	  		arg = args[i++];
			//System.out.println(arg);
			// use this type of check for "wordy" arguments
	       if (arg.equals("-verbose")) {
	            System.out.println("Verbose mode on");
	            VERBOSE = true;
	            }

	       if (arg.equals("-nogui")) {
	            if (VERBOSE) {System.out.println("Batch mode on");}
	            NOGUI = true;
	            }
	            
	       if (arg.equals("-unsteg")) {
	            if (VERBOSE) {System.out.println("Unsteg mode on");}
	            UNSTEG = true;
	            }

       
         // use this type of check for arguments that require arguments
          else if (arg.equals("-image")) {
              if (i < args.length)
                  imagefile = args[i++];
              else
                  System.err.println("-image always requires a URL after it");
              if (VERBOSE) {System.out.println("image file = " + imagefile);}
          }
          
          else if (arg.equals("-input")) {
          	  if (UNSTEG) System.err.println("-input not needed for unstegging");
              if (i < args.length)
                  inputfile = args[i++];
              else
                  System.err.println("-input always requires a file name");
              if (VERBOSE) {System.out.println("input file = " + inputfile);}
          }

          else if (arg.equals("-output")) {
              if (i < args.length)
                  outputfile = args[i++];
              else
                  System.err.println("-output always requires a file name");
              if (VERBOSE) {System.out.println("output file = " + outputfile);}
          }
          
          else if (arg.equals("-filter")) {
              if (i < args.length)
                  filtername = args[i++];
              else
                  System.err.println("-filter requires a class name");
              if (VERBOSE) {System.out.println("filter = " + filtername);}
          }
		} //while
		
	if (args.length != 0)
		{
		if  (i == args.length)
          		 System.err.println("Usage: EzStego -nogui [-verbose] [-unsteg] -image filename -input filename [-filter classname] -output filename");
       		else
         	 if (VERBOSE) {System.out.println("Arguments successfully read.");}
		}
	}

	/* main */
	public static void main(String argv[])
	{
		parseCmdLine(argv);
		stego = new EzStego();
		stego.start();	
    	if (NOGUI)
		{
			stego.init_nogui();
		}

		else
		{
			StegoFrame f = new StegoFrame(versionString);
			f.add("Center",stego);
			f.resize( 400, 400);
			f.show();
		}
	}
	
	public void start()
	{
		super.start();
		//preload all classes to avoid Win95 FileDialog bug
		cDynamicFilterInputStream = new DynamicFilterInputStream();
		cEndsWithFilter = new EndsWithFilter();
		cEzStegoEncoder = new EzStegoEncoder(null,null);
		cRGBPaletteSorter = new RGBPaletteSorter();
		
	}
	
	public void init_nogui()
	{
		openImageFile();
		if (UNSTEG) unSteg();
			else doSteg();
	}

	public Image image()
	{
		return image;
	} 
	
	public Image splashImage()
	{
		return splashImage;
	} 

	static public EzStego app()
	{
		return stego;
	}
	
	public void setStatus( String s, boolean error_State)
	/** Displays status strings */
	{
	if (NOGUI)
	    { 
	    if (VERBOSE) {System.out.println(s);}
	     else if (error_State) {System.err.println(s);}
	    }
		else
		{
			if (error_State) {statusLabel.setForeground(errColor);}
				else {statusLabel.setForeground(foreColor);}
			statusLabel.setText(s);
			repaint();
		}
	}
	


	public boolean action(Event ev, Object arg)
	{
		/**
		 * Handles button presses.
		 * @return           boolean = true if we deal with the event.
		 */

		if (ev.target instanceof Button) {

			if (ev.target == stegButton) {
				doSteg();
				return true;
			}

			if (ev.target == unstegButton) {
				unSteg();
				return true;
			}

			if (ev.target == openGIFButton) {
				openImageFile();
				return true;
			}
			
			if (ev.target == urlButton) {
				fetchImageFile(urlTextField.getText());
				return true;
			}


		}

		return super.action(ev,arg);
	}

	public Image loadImage(File thisImage)
	/** Loads image given the file it's in. Returns null if it can't get image. */

	{
		imageFileString = thisImage.getPath();
		Image theImage = null;
		try
		{
				theImage = createImage(new FileImageSource(imageFileString));
				//Loads the color table into rgbPalette.
				waitForImage(this,theImage);
	
		}
		catch (Throwable e)
		{
			setStatus(" Could not load image",true);
			//Make this bail
			return null;
		}
		getImageStats(theImage);
		
	
		return theImage;
	}
	
	
	
	public void getImageStats(Image theImage)
	{
	  try
		{
				pro = theImage.getSource();
       			pro.addConsumer(this);
        		pro.startProduction(this);

		}
		catch (Throwable e)
		{
				setStatus(" Could not load image ",true);
			//Make this bail
			return;
		}
		while (numRGBvalues == 0) {}; //block until we have the stats!!

	}
	
	
	void waitForImage(Component component, Image myImage)
	{
		MediaTracker tracker = new MediaTracker(component);
		boolean getError = false;
		try
		{
			tracker.addImage(myImage, 0);
			tracker.waitForID(0);
			getError = tracker.isErrorAny();
		} 
		catch (Throwable anyDamnThing)
		{
			setStatus(" Could not load image: "+imageFileString,true);
		}
		if (getError) 
		{
			setStatus(" Could not load image: "+imageFileString,true);
			image = null;
		}
		
	}


	
	// The following methods implement the ImageConsumer interface

    public void imageComplete (int status) {
        // Disconnect this ImageConsumer from the ImageProducer 
        pro.removeConsumer(this);
    }
    
    private void notifyObservers (int flags) {
    }
    
    public void setColorModel (ColorModel model) {
        if (model instanceof IndexColorModel) {
            // Print contents of IndexColorTable
            IndexColorModel im = (IndexColorModel) model;
            numRGBvalues = im.getMapSize();
 
            rgbPalette = new int[numRGBvalues];
         	//if (DEBUG){  System.out.println("Number of color entries = " + numRGBvalues);}
            for (int ii = 0; ii < numRGBvalues; ii++) {
            	rgbPalette[ii] = im.getRGB(ii);
             	//if (DEBUG){ System.out.println(" Index " + ii + " = RGB:" + im.getRGB(ii));}
             	} 
           }
        }
    
    public void setDimensions (int width, int height) {
    }
    
    public void setHints (int hintflags) {
    }
    
    public void setPixels (int x, int y, int w, int h, ColorModel model,
                             byte pixels[], int off, int scansize) {
    }
    
    public void setPixels (int x, int y, int w, int h, ColorModel model,
                             int pixels[], int off, int scansize) {
    }
    
    public void setProperties (Hashtable props) {
    }
	//End ImageConsumer Implementation



    public void run()
    {
    }

    public void stop()
    {
        super.stop();
    }
    
    public void fetchImageFile(String urlString)
    {
    	URL imageURL = null;
    	URLConnection imageURLC = null;
    	setStatus("Fetching "+urlString,false);
	    try
		    {
					imageURL = new URL(urlString);
					imageURLC = imageURL.openConnection();
			}
		catch (Exception e) 
			{
					 setStatus( "Could not construct URL: "+urlString,true);
					 return;
			 }
		 
   		try
	   		{
					String mimetype = imageURLC.getContentType();
					
					if (! (mimetype.equals("image/gif")))
					{
						setStatus( "URL "+imageURL.getFile()+" does not refer to a GIF image.",true);
						return;
					}
					Object content = imageURLC.getContent();
					image = createImage((ImageProducer) content);
					waitForImage(this,image);	

		 	}
		catch (Exception e) 
			{
					 setStatus( "Could not download image: "+imageURL.getFile(),true);
					 return;
			}
		
		
		if (image == null)
		{
			setStatus( "Unable to load image: "+imageURL.getFile(),true);
			return;
		}
		
		getImageStats(image);
		spaceAvail = (image.getHeight(this) * image.getWidth(this))/8;
		setStatus("File: "+imageURL.getFile()+" Space available: "+spaceAvail+" bytes",false);
		if (!NOGUI) 
			{ 
				canvas.setImage(image);
				invalidate();
				getParent().layout();
				canvas.repaint();
				repaint();
			}
		}


    public void openImageFile()
	{
		if (!NOGUI)
		{	
			setStatus( "Select a GIF image file.",false);
			Container f = getParent();

			while (f != null && !(f instanceof Frame)) f = f.getParent();
			FileDialog myfileDialog = new FileDialog((Frame)f,"Open GIF File",FileDialog.LOAD);
			EndsWithFilter gif_filter = new EndsWithFilter(".GIF");
			try
			{
			    myfileDialog.setFilenameFilter(gif_filter);
			}
			catch(Throwable anyDamnThing)
			{
			    // If we can't setFilenameFilter, life goes on.
			}
			myfileDialog.show();
			repaint();
			   

			// If user cancels from dialog without picking a file, return
			if (myfileDialog.getFile() == null) return;
			imagefile = myfileDialog.getDirectory() + myfileDialog.getFile();
			if (imagefile.endsWith(".*.*"))
        		{imagefile = imagefile.substring(0, (inputfile.length() - 4));}


		}
		
		File theImage = new File(imagefile);
		setStatus( "Loading image file.",false);
		image = loadImage(theImage);
		if (DEBUG) {System.out.println("Image file loaded.");}

		if (image == null)
		{
			setStatus( "Unable to load image.",true);
			return;
		}
		
		if (!NOGUI) 
			{ 
				canvas.setImage(image);
				invalidate();
				getParent().layout();
				canvas.repaint();
				repaint();
			}
			
		spaceAvail = (image.getHeight(this) * image.getWidth(this))/8;
		setStatus("File: "+theImage.getName()+" Space available: "+spaceAvail+" bytes",false);

		}
		
	
	
	InputStream getFiletoHide() 
	{
		if (!NOGUI)
		{
			setStatus("Select a file to hide, less than " +spaceAvail+" bytes.",false);

			Container f = EzStego.app().getParent();
			while (f != null && !(f instanceof Frame)) f = f.getParent();

		
			FileDialog myfileDialog = new FileDialog((Frame)f,"",FileDialog.LOAD);
			myfileDialog.show();
			repaint();
			   //Workaround Win95 FileDialog bug
		
			/** If user cancels from dialog, return */
			if (myfileDialog.getFile() == null) return null;
			inputfile = myfileDialog.getDirectory()+myfileDialog.getFile();
			/** If there's a ".*.*" appended here due to Win95 Java  bug  get rid of it */
			if (inputfile.endsWith(".*.*"))
				 {inputfile = inputfile.substring(0, (inputfile.length() - 4));}
			}
				
		File source_file = new File(inputfile);
	
	
		// If there isn't enough room for this file, bail
		if (spaceAvail < (int) source_file.length())
		 {
			setStatus("This file will not fit! " +
			"Available: "+spaceAvail+" bytes. File length: "+(int) source_file.length()+" bytes.",true);
			return null;
		}
				
		InputStream myFileInputStream =  null;
				
		try
		{
			myFileInputStream =  new FileInputStream(source_file);
		}
		catch (Exception e)
		{
		setStatus("Could not open input stream.",true);
		return null;
		}
		
		///Set up DynamicFilterInputStream here
		DynamicFilterInputStream _Filter = null;				
		if (filtername != null)
		{
			try
			{
				 _Filter =
				 DynamicFilterInputStream.newFilter( filtername, myFileInputStream );
				 //will be
				 // _Filter =
				 // DynamicFilterInputStream.newInstance( filtername, myFileInputStream );
			}
			catch (Exception e)
			{
				setStatus("Could not open specified filter class.",true);
				return null;
			}
			
			if (_Filter != null) myFileInputStream = _Filter;
		}
		
		return myFileInputStream;
	}

	public void doSteg()
	{
		if (!NOGUI) 
		{
			//Exit if there's no .GIF currently displayed
			if (image == null)
			{
				setStatus("Please open a GIF file.",true);
				repaint();
				return;
			};
			setStatus("Enter a name for the new GIF file.",false);
			repaint();
			
			Container f = getParent();
			while (f != null && !(f instanceof Frame)) f = f.getParent();

			FileDialog myfileDialog = new FileDialog((Frame) f,"Save Stegged GIF As",FileDialog.SAVE);
			myfileDialog.setFile("output.gif");
			myfileDialog.show();
			repaint();
			   //Workaround Win95 FileDialog bug
 
			// If user cancels from dialog, return 
			if (myfileDialog.getFile() == null) return;
			
			outputfile = myfileDialog.getDirectory()+myfileDialog.getFile();
 		 if (outputfile.endsWith(".*.*"))
        		{outputfile = outputfile.substring(0, (outputfile.length() - 4));}

		}			

		InputStream myFileInputStream = getFiletoHide();
		if (myFileInputStream == null) return;

		// Create output file & try to open a stream to it 
		setStatus("Opening output file stream. ",false);
		OutputStream destination = System.out; //Output defaults to System.out
		File destination_file = null;
		if (outputfile != null)
		{
			destination_file = new File(outputfile);
			try
			{
				destination = new FileOutputStream(destination_file);
			}
			catch (Exception e)
			{
				setStatus("Could not open output file. "+e.getMessage(),true );
				return;
			}
		}
		setStatus("Initizialing image encoder. ",false);
	    try {
				ie = new EzStegoEncoder( image, destination, null, rgbPalette);
				
		} catch (Exception e)
		{
			setStatus( "Could not initialize image encoder."+e.getMessage(),true );
		}
		
		setStatus("Saving new image file. ",false);
	
		// Try to encode the .gif 
		ie.setFunction(EzStegoEncoder.STEG);
		ie.setInputStream(myFileInputStream);

		try
		{
			ie.encode();
		}
		catch (Throwable e)
		{
			setStatus("Stegging failed. Please try a different image file. "+e.getMessage(),true);
			return;
		}
		
		setStatus("Deleting old image file. ",false);
		File oldImageFile = new File(imageDirString,imageFileString);
		boolean filegone = oldImageFile.delete();
		setStatus("Done. ",false);
		
		if (!NOGUI)
		{
		 	setStatus("Loading new image file. ",false);
			image = loadImage(destination_file);
			spaceAvail = (image.getHeight(this) * image.getWidth(this))/8;
			setStatus("File: "+destination_file.getName()+" Space available: "+spaceAvail+" bytes",false);
		}
	}

	public void unSteg()
	{
		if (!NOGUI)
		{
			//if there's no image displayed, get out!
			if (image == null)
			{
				setStatus("Please open a GIF file.",true);
				repaint();
				return;
			};
			// Show dialog and ask for output file
			setStatus("Enter a name for the output file.", false);

			Container f = getParent();
			while (f != null && !(f instanceof Frame)) f = f.getParent();
			FileDialog myfileDialog = new FileDialog((Frame) f,"Save Output File As",FileDialog.SAVE);
			myfileDialog.setFile("output.txt");
			myfileDialog.show();
			repaint();
			   //Workaround Win95 FileDialog bug

			//If user cancels from dialog, return
			if (myfileDialog.getFile() == null) return;
			outputfile = myfileDialog.getDirectory()+myfileDialog.getFile();
			 		 if (outputfile.endsWith(".*.*"))
        		{outputfile = outputfile.substring(0, (outputfile.length() - 4));}


		}
	
		/** Create output file & try to open a stream to it, defaults to System.out */
		OutputStream destination = System.out;
		if (outputfile != null)
		{
			File destination_file = new File(outputfile);
			try
			{
				destination = new FileOutputStream(destination_file);
			}
			catch (IOException e)
			{
				setStatus("Error while saving file. "+e.getMessage(),true);
				return;
			}
		}
		
	    try
	    {
			ie = new EzStegoEncoder( image, destination, null, rgbPalette);
		}
		catch (Exception e)
		{
			setStatus( "Could not initialize image encoder."+e.getMessage(),true);
			return;
		}

		/** Try to encode the .gif */
		ie.setFunction( EzStegoEncoder.UNSTEG);

		try
	    {
			ie.encode();
		}
		catch (IOException e)
		{
			setStatus( "Stego recovery  failed: "+e.getMessage(),true);
			return;
		};
	}
	

	
	void setHorizontalScrollbar(int value, int visible, 
				int min, int max) {
	horizontalSB.setValues(value, visible, min, max);
    }

    void setVerticalScrollbar(int value, int visible, 
				int min, int max) {
	verticalSB.setValues(value, visible, min, max);
    }

    int getHorizontalValue() {
	return horizontalSB.getValue();
    }

    int getVerticalValue() {
	return verticalSB.getValue();
    }


};

class StegoFrame extends Frame {

	public StegoFrame(String aString){
		super(aString);
	}

//
// -- Overrides --
//
        /**
        <PRE>
        Handle Events for the Standalone window.
        Handles:
            Event.WINDOW_DESTROY
        @param anEvent the event to handle
        @see Event.WINDOW_DESTROY
        </PRE>
        */
        public synchronized boolean handleEvent( Event a_Event )
        {
                if ( a_Event.id == Event.WINDOW_DESTROY )
                {
                    Runtime.getRuntime().exit(0);
                }

                return super.handleEvent( a_Event );
        }



};


class EndsWithFilter implements FilenameFilter {
	/** A simple filename filter class.
	It checks the extension. It's case-insensitive. */
	private String extension;

	public EndsWithFilter(String extension) {
		this.extension = extension.toUpperCase();
	}
	
	public EndsWithFilter() {
	}

	public boolean accept(File dir,String name) {
		name = name.toUpperCase();
		if (name.endsWith(extension)) return true;
		else return (new File(dir, name)).isDirectory();
	}
};


class StegoPanel extends Panel
{
	StegoPanel()
	{
	super();
	}
	
    
   public void paint(Graphics g)
	{

		Graphics  panel_g =  getGraphics();
		Dimension d = size();
		int thick = 3;int x = (thick/2);int y = (thick/2);
		Color fillColor =  getBackground();
		Color brightColor = fillColor.brighter().brighter().brighter();
		Color darkColor = fillColor.darker().darker().darker();
		
		g.setColor(brightColor);
		for (int i = 0; i< (thick/2); ++i)
			g.drawRect(x+i,y+i,(d.width - thick),(d.height - thick));
		
		g.setColor(darkColor);
		for (int i = 0; i< (thick/2); ++i)
			g.drawRect(x+(thick/2)+i,y+(thick/2)+i,(d.width - thick),(d.height - thick));
	}
}
    

class StegoCanvas extends Canvas
{
	Image curImage = null;
	
	public StegoCanvas()
	{
		super();
	}
	
	public void setImage(Image i)
	{
	curImage = i;
	}
	
	public void paintSplashInfo(Graphics g)
	{
		Graphics canvas_g = getGraphics();
		Dimension d = size();
		FontMetrics fm = getFontMetrics(getFont());

		String label = "EzStego";
		
		int lx = labelX(label);
		int ascent = fm.getAscent();
		int spacing = 5;
		int ly = (ascent + spacing);
		
		canvas_g.drawString(label,lx,ly);
		
		ly = ly + ascent + spacing;
		label = "a simple steganography utility";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);
		
		ly = ly + ascent + spacing;
		label = "EzStego is shareware.";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);

		ly = ly + ascent + spacing;
		label = "If you find it useful please send $20 to:";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);

		ly = ly + ascent + spacing;
		label = "Romana Machado, 19762 Stevens Creek Blvd. #127";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);
		
		ly = ly + ascent + spacing;
		label = "Cupertino, CA 95014 USA";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);
		
		ly = ly + ascent + spacing;
		label = "Help, tips, license and source are available at:";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);
		
		ly = ly + ascent + spacing;
		label = "http://www.fqa.com/ezstego/";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);

		ly = ly + ascent + spacing;
		label = "Thanks to software gurus Geoff Dale and Jeff Poskanzer";
		lx = labelX(label);
		canvas_g.drawString(label,lx,ly);

	}
	
	int labelX(String lstring) //Centers the labelstring
	{
	Dimension d = size();
	FontMetrics fm = getFontMetrics(getFont());
	int x = (d.width/2) - (fm.stringWidth(lstring)/2);
	return x;
	}

	public void paint(Graphics g)
	{
		int x, y;
		int imageWidth, imageHeight;
    		int top = 0, left = 0;
    		int width = 200, height = 200;

		x = EzStego.app().getHorizontalValue();
		y = EzStego.app().getVerticalValue();


		Dimension d = size();
		
		if (curImage != null)
		{
			try
			{	
				imageHeight = curImage.getHeight(this);
				imageWidth = curImage.getWidth(this);
				g.drawImage(curImage, -x,-y, imageWidth,imageHeight, Color.black, this);

				height = this.size().height;
				width = this.size().width;

				EzStego.app().setHorizontalScrollbar(x, width,
							0, imageWidth - width);
				EzStego.app().setVerticalScrollbar(y, height,
							0, imageHeight - height);

			}
			catch (Exception e)
			{
				EzStego.app().setStatus("Error drawing image file: "+e.getMessage(),true);
			}
		}
		
		else
		{
		paintSplashInfo(g);
		}
		

		
	}
};

class ImageScrollbar extends Scrollbar {

    int lineNo = 0;
    static int pagesize = 10;
    static int MAX = 100;
    Canvas canvas;

    ImageScrollbar(int o, int v, int vis, int min, int max, 
		Canvas c) {
	super(o, v, vis, min, max);
	canvas = c;
    }

    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	    case Event.SCROLL_LINE_DOWN:
			scrollLineDown();
			break;
	    case Event.SCROLL_LINE_UP:
			scrollLineUp();
			break;
	    case Event.SCROLL_PAGE_DOWN:
			scrollPageDown();
			break;
	    case Event.SCROLL_PAGE_UP:
			scrollPageUp();
			break;
	    case Event.SCROLL_ABSOLUTE:
			scrollAbsolute();
			break;
	}
	return false;
    }

    protected void scrollLineUp() {
	canvas.repaint();
    }

    protected void scrollLineDown() {
	canvas.repaint();
    }

    protected void scrollPageUp() {
	canvas.repaint();
    }

    protected void scrollPageDown() {
	canvas.repaint();
    }

    protected void scrollAbsolute() {
	canvas.repaint();
    }
}

