import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import ImageEncoder;
import GifEncoder;
import RGBPaletteSorter;


public  class EzStegoEncoder extends GifEncoder{
	final static int STEG = 1;
	final static int UNSTEG = 2;

	private InputStream stegInputStream = null;
	private boolean DEBUG = false;
	private int spaceAvail;
	int[] newPalette;
	int[] rgbPalette;
	int numRGBvalues;
	int bitDepth;
	byte[] reds;  //Create a new array of 256 bytes for each of reds, greens, and blues
	byte[] grns; 
	byte[] blus;
	int numPixels; 

	/** Stuffs (stegs) a file, into a GIF File */

	//Constructor.
	public EzStegoEncoder(Image img, OutputStream out, 
	FileInputStream in, int[] rgbPalette )
	{
		super( img, out );
		stegInputStream = in;
		this.rgbPalette = rgbPalette;
		this.numRGBvalues = rgbPalette.length;
		this.bitDepth = bitDepth;

	}

	//Constructor.
	public EzStegoEncoder(ImageProducer prod, OutputStream out )
	{
	super(prod, out);
	}

	public EzStegoEncoder( ImageProducer prod, OutputStream out,
	InputStream in,  int[] rgbPalette )
	{
		super( prod, out);
		stegInputStream = in;
		this.rgbPalette = rgbPalette;
		this.numRGBvalues = rgbPalette.length;
		this.bitDepth = bitDepth;

	}
	public static boolean bitTest(int tbyte, int bitnum)
	{
		int mybyte = 0x80;
		mybyte = mybyte >>> bitnum;
		mybyte = mybyte & tbyte;
		if (mybyte != 0) return true;
		return false;
	}

	public static byte bitSet(int tbyte, int bitnum)
	{
		int mybyte = 0x80;
		mybyte = mybyte >>> bitnum;
		mybyte = mybyte | tbyte; 
			return (byte) mybyte;
	}

	public static byte bitClear(int tbyte, int bitnum)
	{
		int mybyte = 0xFF; 
		mybyte = mybyte ^ (0x80 >>> bitnum);
		mybyte = mybyte & tbyte; 
			return (byte) mybyte;
	}
	
	
	 int getIndexFromRGB(int rgb,int[] palette)
	{
		int index = -1;
		for (int n = 0; n<numRGBvalues; n++)
		{
			if (palette[n] == rgb) {index  = n;  break;}
		}
		///
		if (index == -1) 
			{
			System.err.println("Color "+rgb+" not found!");
			}
		return index;
	}


	void encodeStego() throws IOException
		{
		showMessage(" Creating new GIF image...");

		//Steganographic encoding starts here /
		int bytes_read;
		byte[] buffer = new byte[1024];
		int i = 0;
		byte b = 0;
		boolean noErr = true;

		stegger: while (true)
		{
			//Progress indicator
			if  ((i% 100) == 0) {showMessage("Stegging byte #"+(i/8));}

			bytes_read = stegInputStream.read(buffer);
			if (bytes_read == -1) break;
			for (int n = 0; n < bytes_read; n++)
			{
				 b = buffer[n];
				for (int bit = 0; bit < 8; bit++)
				{
					int RGBidx = getIndexFromRGB(rgbPixels[i],newPalette);
					if (RGBidx == -1) {  	//Error condition
						            noErr = false;
					                    break stegger;
				 	}
				 	
					if (bitTest(b,bit)) {
						RGBidx = bitSet((byte) RGBidx, 7);
						if (DEBUG) {if  (bitTest((byte) RGBidx,7)) {System.out.print("1");}}
					}
					else {
						RGBidx = bitClear((byte) RGBidx, 7);
						 if (DEBUG) {if ( !bitTest((byte) RGBidx, 7)) {System.out.print("0");}}
					}
					
					int newRGB = newPalette[RGBidx & 0xFF]; 
					int newRgbidx = getIndexFromRGB(newRGB, rgbPalette);  
					rgbPixels[i] = rgbPalette[newRgbidx];
					i++;
	
				}//for bit
			}//for n
		}//for j
		//Save the .GIF
		if  (noErr) 
			{
				showMessage("Saving new GIF image...");
				///calculate bitDepth
				int n = rgbPalette.length-1; 
				int bitDepth = 0;
				while (n>0) {n = n/2; bitDepth++;}
				//showMessage("Bit Depth ="+bitDepth);
				super.GIFEncode(out, width, height, interlace, (byte) 0, -1, bitDepth, reds, grns, blus );
			}
	    else 	//throw an exception
	    		{
	    			throw new IOException("Color not found: "+rgbPixels[i]);
	    		}
	}
	
	
	void showProgress(String message, int i)	//Progress indicator
	{
	  if ( (i % 100) == 0)
				{
				double pixelsRemaining =  (numPixels - i);
				double percentComplete = 100.0 * (1.0 - (pixelsRemaining / numPixels));
				int pc = (int)percentComplete;
				EzStego s = EzStego.app();
				s.setStatus(message+pc+"%", false);
				}
	}
			
	void showMessage(String message)	//Progress indicator
	{
		EzStego s = EzStego.app();
		s.setStatus(message,false);
	}


	void decodeStego() throws IOException
		{
		showMessage("Unstegging from .GIF");

		/** Steganography decoding starts here */
		if (DEBUG) {
			System.out.println("In decodeStego:  numPixels "+numPixels);}
		int i = 0;	
		int b = 0;
		int n = 0;
		
		byte[] outByteBuf = new byte[1024];
		
		while (i < (numPixels - 1)) //get data from all pixels in the image
		{
				for (n = 0; n<1024; n++) //fill output buffer
			{
				outByteBuf[n] = 0; //clear 
				for (int bit = 0; bit < 8; bit++)
					{
						b =  getIndexFromRGB(rgbPixels[i],newPalette);
						if (bitTest((byte)b,7))
						 { 
							outByteBuf[n] = bitSet(outByteBuf[n], bit);
							if (DEBUG) {if (bitTest(outByteBuf[n], bit)) {System.out.print("1");}}
						}
						else
						{
							outByteBuf[n] = bitClear(outByteBuf[n], bit);
							if (DEBUG) {if  (!bitTest(outByteBuf[n], bit)) {System.out.print("0");}}
						}
						i++;
						if  (i == numPixels) break;
					}
				if  (i == numPixels) break;
				}
			showProgress("Writing to output file ",i);
			out.write(outByteBuf);
		}
		
		out.close(); 
		showMessage("Done. Output file saved.");

	}
	
	void setInputStream(InputStream f)
	{
	stegInputStream = f;
	}
	
	
	void sortPalette()
	{
	
	showMessage("Sorting palette.");
	RGBPaletteSorter ps = new RGBPaletteSorter(rgbPalette);
	//System.out.println("Before:");
	//ps.dumpPalette();
	ps.orderPalette();
	//System.out.println("After:");
	//ps.dumpPalette();
	newPalette = ps.getPalette();
	}
	
	
	private int do_it;
	
	public void setFunction(int do_this)
	{
	do_it = do_this;
	}
	
	void encodeDone() throws IOException
		{
		sortPalette(); 
		
		showMessage("Preparing image.");
		encodePrep();
		
		numPixels = width * height;
		if (do_it == STEG)
		 {
			encodeStego();
		}
		else
		 {
		 	//makeStegArray();

			decodeStego();
		}
	}
	
 void encodePrep() throws IOException
     {

	//We will use these three arrays to pass the color map to GIF Encode
 	reds = new byte[256]; 
	grns = new byte[256];
	blus = new byte[256];

	
  	for( int i = 0; i < numRGBvalues; ++i)  
	  {
	  // if (DEBUG) {System.out.println(i+" "+  rgbPalette[i] );}
	    reds[i] = (byte) ( ( rgbPalette[i] >> 16 ) & 0xff ); 
	    grns[i] = (byte) ( ( rgbPalette[i] >>  8 ) & 0xff );
	    blus[i] = (byte) (  rgbPalette[i]   & 0xff ); 
	   } 	
  }

	
    byte GetPixel( int x, int y ) throws IOException
	{
	return (byte) getIndexFromRGB( rgbPixels[y * width + x] , rgbPalette);
	}
	
	// Return the next pixel from the image
    int GIFNextPixel() throws IOException
	{
	int r;
	if ( CountDown == 0 )
	    return EOF;

	--CountDown;

	r = (GetPixel( curx, cury ) & 0xFF);
	BumpPixel();

	return r;
	}

};

