//import java.awt.Color;

/**
Utility for ordering RGB palettes
*/
public class RGBPaletteSorter
{	
	//private int[][] distances;
	private int[]  inPalette;       //a local copy of the disorderly palette
	private int[] 	fCycle;
    private int[] 	fDist; // distances to all other colors from current color.
	private int fIteration; 		//The current iteration of the path minimizing algorithm
	private int paletteSize;
	private int fStartColor;		// Starting color for the path
	private int 	minDistance;
	//private int pSizeMinusOne;
	
	/**
	Initialize inPalette, Mask off alpha for optimization.
	@param startPalette Palette to be sorted
	*/
	public RGBPaletteSorter(int[] startPalette)
	{
	    this.paletteSize = startPalette.length;
		//this.pSizeMinusOne = paletteSize - 1;
		this.inPalette = startPalette; 
	}
	
	
	public RGBPaletteSorter()
	{
	}

	/*
		Return with alpha added back in
	*/
	public int[] getPalette()
	{
		for (int n = 0; n < paletteSize; n++)
		{
			inPalette[n] = 0xFF000000 | inPalette[n];
		}
	
		return inPalette;
	}
	
	
	/**
	Debug Utility: Dump current palette to System.out.
	*/
	public void dumpPalette() 
	{
		for (int n = 0; n < paletteSize; n++)
		{
			System.out.println(n+": "+ (0xFF000000 | inPalette[n]) );
		}
	}

	/**
	Calculates the total path distance of the PaletteSorter object
	which is the length of the traveling salesman's "tour" through the palette.
	
	@returns total path distance thru color space
	*/
	public final int getTotalRGBPathDistance()
    {
        int totalDistance = 0;
    	for (int i = 1; i<paletteSize; i++)
    	{
    		totalDistance += DistanceRGBs(inPalette[i-1],inPalette[i]);
    	}
    	totalDistance += DistanceRGBs(inPalette[(paletteSize-1)],inPalette[0]);
    	return totalDistance;
    }

	
	/**
	Calculates distance between 2 colors in 3d color space.
	
	@returns path distance thru color space
	*/
	public final int DistanceRGBs(int rgb1, int rgb2)
	{
		int d_Red   = (( rgb1 & 0xFF0000) - (rgb2 & 0xFF0000)) >> 16;
		int d_Green = (( rgb1 & 0x00FF00) - (rgb2 & 0x00FF00)) >> 8;
		int d_Blue = ( rgb1 & 0x0000FF) - (rgb2 & 0x0000FF);
		return ((d_Red * d_Red) + (d_Blue * d_Blue) + (d_Green * d_Green));
	}

	
/*	private final void initDistances()
	{
		for (int i = 0; i<= pSizeMinusOne; i++)
        {
         	distances[i][i] = 0;
            for(int j = i+1;j<paletteSize;j++)
            {
            	distances[j][i] = distances[i][j] = DistanceRGBs(inPalette[i],inPalette[j]);
            }
            distances[pSizeMinusOne][pSizeMinusOne] = 0;
        }
		//System.out.println("Initialized distance matrix");
	}
	*/
	
    /**
    index = current color index (inPalette) 
    value = next color index (inPalette).
    */
	
	/**
	Palette ordering algorithm
	*/
	public void orderPalette()
	{
		int 	nextColor;
        int[]  	outPalette = new int[paletteSize];  //the ordered palette
        
		//distances = new int[paletteSize][paletteSize];
		//initDistances();

		//Init Insertions
		fStartColor = 0; //findStartColor();
		
		fCycle = new int[paletteSize];
		fDist = new int[paletteSize];
		for (int i = 0; i < paletteSize; i++) 
		{
			// fCycle initialized to -1 indicating the next color is unknown.
			fCycle[i] = -1;
			// fDist initialized to distances to all other colors from start color.
			fDist[i] = DistanceRGBs(inPalette[0],inPalette[i]);
		}
		fCycle[0] = fStartColor;
			
		minDistance = 0;
			
		//Repeat Insertions		

		for (int i = 1; i < paletteSize; i++)  //Put remaining elements in order
		{
			fIteration = i;
			nextColor = findNextColor();
			insertColor(nextColor);
			
			//Debugging code
			
			//if ((i%256) == 0) System.out.println("After inserting "+i+" colors");
			/*if (i > (paletteSize - 3))
			{
				System.out.println("After inserting "+nextColor+" = "+inPalette[nextColor]);
				int k;
	        	k = fStartColor;
	        	for (int x = 0; x < fIteration; x++)
	        	{
				
	                    System.out.print(x+":"+k+" ");
	                    k = fCycle[k];
	                    if (((x+1) % 8) == 0) {System.out.println(" ");}
	         	}
				System.out.println(" ");
				System.out.println(" ");
			}*/
			
			// Initialize for the next iteration
			for (int j = 0; j<paletteSize; j++)
			{
				if (fCycle[j] == -1)
				{
					//if  ( distances[nextColor][j] < fDist[j] )
					//	fDist[j] = distances[nextColor][j];
					int test  = DistanceRGBs(inPalette[nextColor],inPalette[j]);
					if (test < fDist[j]) fDist[j] = test;
				}
			}

		}
			
		int index;
        index = fStartColor;
        for (int i = 0; i< paletteSize; i++)
        {
            outPalette[i] = inPalette[index];
            index = fCycle[index];
        }
          
       inPalette = outPalette;
 
	}
	
	/**
	Finds farthest unused color from current color.
	*/
	private final int findNextColor() //uses fCycle, fDist ,paletteSize                      
    {
        int MaxDist = Integer.MIN_VALUE;
        int farthest = 0;
        
        for (int j = 0; j < paletteSize; j++)
        {
            if (fCycle[j] == -1)
            {
             	if (fDist[j] > MaxDist) 
                {
					MaxDist = fDist[j];
                	farthest = j;
                 }
             }
        }
     	return farthest;
    }

	
	private final void insertColor (int nextColor)
    {
		int insCost, newCost;
		int index, nextIndex, end1, end2, j;
		            
		insCost = Integer.MAX_VALUE;
		index = fStartColor;
		end2 = 0;
		end1 = 0;
		
	    for (j = 0; j <= fIteration; j++)  //For each color in the current iteration
	    								  //or "tour"
	    {
	    	nextIndex = fCycle[index]; //Get the next index (starting at the starting color)
	    	int d1 = DistanceRGBs(inPalette[index],inPalette[nextColor]);
	    	int d2 = DistanceRGBs(inPalette[index],inPalette[nextIndex]);
	    	
	    	if (index != nextIndex)  //Calculate the "cost" of inserting the index color
		    {
		     //	newCost = distances[index][nextColor] + 
		     //	 distances[nextColor][nextIndex]
	         //  		- distances[index][nextIndex];
	         newCost = d1 + DistanceRGBs(inPalette[nextColor],inPalette[nextIndex]) - d2;
	        }
			else
			{
			//	newCost = distances[index][nextColor] + 
			//			distances[index][nextIndex];
			newCost = d1 + d2;
	  		}
	       	
	       	if (newCost < insCost)   //find the lowest insertion cost
	       	{
	           	insCost = newCost;
	           	end1 = index;
	           	end2 = nextIndex;
	        }
	         
	      	index = nextIndex;
		}

		fCycle[nextColor] = end2;
		fCycle[end1] = nextColor;

		minDistance += insCost; //add insCost to the minimum distance
		     
		//System.out.println ("minDistance: "+minDistance);
	} /*insertColor*/
	

}//RGBPaletteSorter
