/**
 * <PRE>
 * Copyright and Permissions:
 *  Copyright Geoff Dale 1996, All rights reserved.
 *  You may freely redistribute this file as long as no modifications are made
 *     to the code or the comments (including this copyright).
 *  You may freely redistribute the compiled .class file created by this file.
 *  You may freely subclass from this class.
 *
 *
 * Naming Conventions:
 *  ClassName
 *  methodName
 *  void setVariableName( type a )  - setter accessor method
 *  type getVariableName()          - getter accessor method
 *  VARIABLE_NAME       - constants (static final)
 *  _VariableName       - (private or protected) instance variable
 *  s_VariableName      - static instance variable
 *  a_VariableName      - method argument
 *  a                   - setter argument
 *  variableName        - public instance or local variable
 * </PRE>
 * @author Geoff Dale <geoff1@home.com>
 * @version 0.1
 */


import java.io.*;

/**
<PRE>
Example DynamicFilterInputStream
</PRE>
*/
public class ToUpperFilterInputStream
    extends DynamicFilterInputStream
{
//
// -- Constructor --
//
    /**
     * No arg constructor so that we may be created by newInstance.
     */
    public ToUpperFilterInputStream()
    {
    }

    /**
     * Reads a byte. Will block if no input is available.
     * @return 	the byte read, or -1 if the end of the
     *		stream is reached.
     * @exception IOException If an I/O error has occurred.
     */
    public int  read()
        throws IOException
    {
	    return (int) Character.toUpperCase( (char) super.read() );
    }

    /**
     * Reads into an array of bytes.
     * Blocks until some input is available.
     * This method should be overridden in a subclass for
     * efficiency (the default implementation reads 1 byte
     * at a time).
     * @param b	the buffer into which the data is read
     * @param off the start offset of the data
     * @param len the maximum number of bytes read
     * @return  the actual number of bytes read. Returns -1
     * 		when the end of the stream is reached.
     * @exception IOException If an I/O error has occurred.
     */
    public int read(
        byte    a_Buffer[],
        int     offset,
        int     len )
        throws IOException
    {
        int bytesRead = super.read( a_Buffer, offset, len );

        for ( int i = offset; i < len; i++ )
        {
            a_Buffer[i] = (byte) Character.toUpperCase( (char) a_Buffer[i]  );
        }

	    return bytesRead;
    }

}